/* 2D Magnet Simulation */

/* To solve: getdp Rot2DDiscMagnets -pre MagSta_phi -cal -pos phi */

Group {
	Magnet1=Region[100];
	Magnet2=Region[110];
	Air=Region[200];

	Domain = Region[{Air,Magnet1,Magnet2}];
	Domain_Air= Region[Air];
	Domain_M1 = Region[Magnet1];
	Domain_M2 = Region[Magnet2];
}

Function {
	mu0 = 4.e-7 * Pi;
	murMag = 1.045;								/* NdFe32MGO2 */

	nu[ Air ]    = 1. / mu0;
	nu[ Magnet1 ] = 1. / murMag * mu0;
	nu[ Magnet2 ] = 1. / murMag * mu0;
	
	mu[ Air ]    = mu0;
	mu[ Magnet1 ] = murMag * mu0;
	mu[ Magnet2 ] = murMag * mu0;

	/* Magnetic Field Vector  in [A/m] */
	hc[ Magnet1 ] = Vector[0. ,883310., 0.];	/* Hc in A/m for NdFe32MGO2 */
	hc[ Magnet2 ] = Vector[883310. , 0., 0.];	/* HC in A/m for NdFe32MGO2 */
}

/* Resolution */

/* Input

  Groups :
  --------
    Domain         Whole magnetic domain
    Domain_M1      Permanent magnet1 regions 
    Domain_M2      Permanent magnet2 regions 
		 
  Functions :	 
  -----------	 
    mu[]           Magnetic permeability
    nu[]           Magnetic reluctivity
    hc[]           Coercitive magnetic field
    
  Constraint :	 
  ----------	 
    phi            Fixed magnetic scalar potential
    a              Fixed magnetic vector potential (2D)

*/


Jacobian {
  { Name JVol ;
    Case { 
      { Region All ;        Jacobian Vol ; }
    }
  }
}


Integration {
  { Name I1 ;
    Case { 
      { Type Gauss ;
        Case { 
	  { GeoElement Triangle    ; NumberOfPoints  4 ; }
	  { GeoElement Quadrangle  ; NumberOfPoints  4 ; }
	}
      }
    }
  }
}

/* --------------------------------------------------------------------------
   MagSta_phi : Magnetic scalar potential phi formulation 
   -------------------------------------------------------------------------- */

FunctionSpace {
  { Name Hgrad_phi ; Type Form0 ;
    BasisFunction {
      { Name sn ; NameOfCoef phin ; Function BF_Node ;
        Support Domain ; Entity NodesOf[ All ] ; }
    }
    Constraint {
      { NameOfCoef phin ; EntityType NodesOf ; NameOfConstraint phi ; }
    }
  }
}


Formulation {
  { Name MagSta_phi ; Type FemEquation ;
    Quantity { 
      { Name phi ; Type Local ; NameOfSpace Hgrad_phi ; }
    }
    Equation {
      Galerkin { [ - mu[] * Dof{d phi} , {d phi} ] ;  
                 In Domain ; Jacobian JVol ; Integration I1 ; }
      Galerkin { [ - mu[] * hc[], {d phi} ];
                 In Domain_M1 ; Jacobian JVol ; Integration I1 ; }
      Galerkin { [ - mu[] * hc[], {d phi} ];
                 In Domain_M2 ; Jacobian JVol ; Integration I1 ; }           
    }
  }
}


Resolution {
  { Name MagSta_phi ;
    System {
      { Name A ; NameOfFormulation MagSta_phi ; }
    }
    Operation { 
      Generate[A] ; Solve[A] ; SaveSolution[A] ; 
    }
  }
}


PostProcessing {
  { Name MagSta_phi ; NameOfFormulation MagSta_phi ;
    Quantity {
      { Name b  ; Value { Local { [ - mu[] * {d phi} ] ; In Domain ; Jacobian JVol ; } 
                          Local { [ - mu[] * hc[] ]    ; In Domain_M1 ; Jacobian JVol ; }
                          Local { [ - mu[] * hc[] ]    ; In Domain_M2 ; Jacobian JVol ; } } }
      { Name h  ; Value { Local { [ - {d phi} ]        ; In Domain_Air ; Jacobian JVol ; } } }
      { Name phi; Value { Local { [ { phi } ]          ; In Domain ; Jacobian JVol ; } } }
    }
  }
}


eps = 1.e-5;

PostOperation {
	{Name phi; NameOfPostProcessing MagSta_phi;
	 	Operation {
			Print[ phi, OnElementsOf Domain, File "phi.pos"];
			Print[ h,   OnElementsOf Domain, File "h_phi.pos", Depth 0 ] ;
			Print[ b,   OnElementsOf Domain, File "b_phi.pos", Depth 0 ] ;
		}
	}
}